@@ -5,12 +5,11 @@ from . import assign_permissions_to_groups
 
 class Migration(migrations.Migration):
     dependencies = [
-        ('openwisp_users', '0004_default_groups'),
         ('config', '0014_device_hardware_id'),
     ]
 
     operations = [
         migrations.RunPython(
             assign_permissions_to_groups, reverse_code=migrations.RunPython.noop
-        ),
+        )
     ]
