@@ -1,14 +1,20 @@
 from hashlib import md5
 from unittest.mock import patch
 
+from django.core.cache import cache
 from django.core.exceptions import ValidationError
+from django.http.response import Http404
 from django.test import TestCase
 from django.urls import reverse
 from swapper import load_model
 
 from openwisp_users.tests.utils import TestOrganizationMixin
-from openwisp_utils.tests import catch_signal
+from openwisp_utils.tests import capture_any_output, catch_signal
 
+from .. import settings as app_settings
+from ..base.config import logger as config_model_logger
+from ..controller.views import DeviceChecksumView
+from ..controller.views import logger as controller_views_logger
 from ..signals import (
     checksum_requested,
     config_download_requested,
@@ -57,7 +63,11 @@ class TestController(
         d = self._create_device_config()
         c = d.config
         url = reverse('controller:device_checksum', args=[d.pk])
-        response = self.client.get(url, {'key': d.key, 'management_ip': '10.0.0.2'})
+        with patch.object(
+            Config, 'get_cached_checksum', return_value=c.checksum
+        ) as mock:
+            response = self.client.get(url, {'key': d.key, 'management_ip': '10.0.0.2'})
+            mock.assert_called_once()
         self.assertEqual(response.content.decode(), c.checksum)
         self._check_header(response)
         d.refresh_from_db()
@@ -69,6 +79,128 @@ class TestController(
         self.assertIsNotNone(d.last_ip)
         self.assertIsNone(d.management_ip)
 
+    def test_device_get_object_cached(self):
+        d = self._create_device_config()
+        view = DeviceChecksumView()
+        view.kwargs = {'pk': str(d.pk)}
+        logger = controller_views_logger
+
+        with self.subTest('check cache set'):
+            with patch('django.core.cache.cache.set') as mock:
+                self.assertEqual(view.get_device(), d)
+                mock.assert_called_once()
+
+        with self.subTest('check cache get'):
+            with patch('django.core.cache.cache.get', return_value=d) as mock:
+                view.get_device()
+                mock.assert_called_once()
+
+        with self.subTest('ensure DB is hit when cache is clear'):
+            with patch.object(logger, 'debug') as mocked_debug:
+                with self.assertNumQueries(1):
+                    self.assertEqual(view.get_device(), d)
+                    mocked_debug.assert_called_once()
+
+        with self.subTest('ensure DB is NOT hit when cache is present'):
+            with patch.object(logger, 'debug') as mocked_debug:
+                with self.assertNumQueries(0):
+                    self.assertEqual(view.get_device(), d)
+                    mocked_debug.assert_not_called()
+
+        with self.subTest('test manual invalidation'):
+            with patch.object(logger, 'debug') as mocked_debug:
+                with self.assertNumQueries(1):
+                    view.get_device.invalidate(view)
+                    self.assertEqual(view.get_device(), d)
+                    mocked_debug.assert_called_once()
+
+        with self.subTest('test automatic cache invalidation'):
+            with patch.object(logger, 'debug') as mocked_debug:
+                d.os = 'test_cache'
+                d.save()
+                mocked_debug.assert_called_once()
+
+            with self.assertNumQueries(1):
+                obj = view.get_device()
+            self.assertEqual(obj.os, 'test_cache')
+
+        with self.subTest('test cache invalidation on device delete'):
+            d.delete()
+            with self.assertNumQueries(1):
+                with self.assertRaises(Http404):
+                    view.get_device()
+
+    def test_get_cached_checksum(self):
+        d = self._create_device_config()
+        # avoid cache to be invalidated by the update of the addresses
+        d.last_ip = '127.0.0.1'
+        d.management_ip = '10.0.0.2'
+        d.save()
+
+        url = reverse('controller:device_checksum', args=[d.pk])
+
+        with self.subTest('first request does not return value from cache'):
+            with self.assertNumQueries(3):
+                with patch.object(
+                    controller_views_logger, 'debug'
+                ) as mocked_view_debug:
+                    with patch.object(
+                        config_model_logger, 'debug'
+                    ) as mocked_config_debug:
+                        self.client.get(
+                            url, {'key': d.key, 'management_ip': '10.0.0.2'}
+                        )
+                        self.assertEqual(mocked_config_debug.call_count, 1)
+                    self.assertEqual(mocked_view_debug.call_count, 1)
+
+        with self.subTest('update_last_ip updates the cache'):
+            with self.assertNumQueries(3):
+                with patch.object(
+                    controller_views_logger, 'debug'
+                ) as mocked_view_debug:
+                    with patch.object(
+                        config_model_logger, 'debug'
+                    ) as mocked_config_debug:
+                        self.client.get(
+                            url, {'key': d.key, 'management_ip': '10.0.0.3'}
+                        )
+                        mocked_config_debug.assert_not_called()
+                    mocked_view_debug.assert_called_once_with(
+                        f'invalidated view cache for device ID {d.pk.hex}'
+                    )
+            view = DeviceChecksumView()
+            view.kwargs = {'pk': str(d.pk)}
+            key = view.get_device.get_cache_key(view)
+            d.refresh_from_db()
+            cached_device = cache.get(key)
+            self.assertEqual(cached_device, d)
+            self.assertEqual(cached_device.management_ip, '10.0.0.3')
+
+        with self.subTest('second request returns value from cache'):
+            with self.assertNumQueries(0):
+                with patch.object(
+                    controller_views_logger, 'debug'
+                ) as mocked_view_debug:
+                    with patch.object(
+                        config_model_logger, 'debug'
+                    ) as mocked_config_debug:
+                        self.client.get(
+                            url, {'key': d.key, 'management_ip': '10.0.0.3'}
+                        )
+                        mocked_config_debug.assert_not_called()
+                    mocked_view_debug.assert_not_called()
+
+        with self.subTest('ensure cache invalidation works'):
+            old_checksum = d.config.checksum
+            with patch.object(config_model_logger, 'debug') as mocked_debug:
+                d.config.config['general']['timezone'] = 'Europe/Rome'
+                d.config.full_clean()
+                d.config.save()
+                del d.config.backend_instance
+                self.assertNotEqual(d.config.checksum, old_checksum)
+                self.assertEqual(d.config.get_cached_checksum(), d.config.checksum)
+                mocked_debug.assert_called_once()
+
     def test_device_checksum_requested_signal_is_emitted(self):
         d = self._create_device_config()
         url = reverse('controller:device_checksum', args=[d.pk])
@@ -101,12 +233,14 @@ class TestController(
                 request=response.wsgi_request,
             )
 
+    @capture_any_output()
     def test_device_checksum_400(self):
         d = self._create_device_config()
         response = self.client.get(reverse('controller:device_checksum', args=[d.pk]))
         self.assertEqual(response.status_code, 400)
         self._check_header(response)
 
+    @capture_any_output()
     def test_device_checksum_403(self):
         d = self._create_device_config()
         response = self.client.get(
@@ -159,6 +293,7 @@ class TestController(
                 request=response.wsgi_request,
             )
 
+    @capture_any_output()
     def test_device_download_config_400(self):
         d = self._create_device_config()
         response = self.client.get(
@@ -167,6 +302,7 @@ class TestController(
         self.assertEqual(response.status_code, 400)
         self._check_header(response)
 
+    @capture_any_output()
     def test_device_download_config_403(self):
         d = self._create_device_config()
         path = reverse('controller:device_download_config', args=[d.pk])
@@ -196,12 +332,14 @@ class TestController(
         )
         self.assertEqual(response.status_code, 404)
 
+    @capture_any_output()
     def test_vpn_checksum_400(self):
         v = self._create_vpn()
         response = self.client.get(reverse('controller:vpn_checksum', args=[v.pk]))
         self.assertEqual(response.status_code, 400)
         self._check_header(response)
 
+    @capture_any_output()
     def test_vpn_checksum_403(self):
         v = self._create_vpn()
         response = self.client.get(
@@ -234,6 +372,7 @@ class TestController(
         )
         self.assertEqual(response.status_code, 404)
 
+    @capture_any_output()
     def test_vpn_download_config_400(self):
         v = self._create_vpn()
         response = self.client.get(
@@ -242,6 +381,7 @@ class TestController(
         self.assertEqual(response.status_code, 400)
         self._check_header(response)
 
+    @capture_any_output()
     def test_vpn_download_config_403(self):
         v = self._create_vpn()
         path = reverse('controller:vpn_download_config', args=[v.pk])
@@ -290,6 +430,24 @@ class TestController(
     def test_register_with_management_ip(self):
         self.test_register(management_ip='10.0.0.2')
 
+    def test_default_template_selection_with_backend_filtering(self):
+        self._create_template(
+            name='t1',
+            backend='netjsonconfig.OpenWisp',
+            organization=self._get_org(),
+            default=True,
+        )
+        t2 = self._create_template(
+            name='t2',
+            backend='netjsonconfig.OpenWrt',
+            organization=self._get_org(),
+            default=True,
+        )
+        d = self.test_register()
+        qs = d.config.templates.all()
+        self.assertEqual(len(qs), 1)
+        self.assertEqual(qs.first().pk, t2.pk)
+
     def test_register_device_info(self):
         device_model_name = 'TP-Link TL-WDR4300 v1'
         os = 'LEDE Reboot 17.01-SNAPSHOT r3270-09a8183'
@@ -312,6 +470,7 @@ class TestController(
         )
         self.assertContains(response, 'already exists', status_code=400)
 
+    @capture_any_output()
     def test_register_failed_creation_wrong_backend(self):
         self.test_register()
         response = self.client.post(
@@ -481,7 +640,7 @@ class TestController(
             )
             d.config.refresh_from_db()
             handler.assert_called_once_with(
-                sender=Config, signal=config_status_changed, instance=d.config,
+                sender=Config, signal=config_status_changed, instance=d.config
             )
         self._check_header(response)
         d.config.refresh_from_db()
@@ -496,7 +655,7 @@ class TestController(
             )
             d.config.refresh_from_db()
             handler.assert_called_once_with(
-                sender=Config, signal=config_status_changed, instance=d.config,
+                sender=Config, signal=config_status_changed, instance=d.config
             )
         self._check_header(response)
         d.config.refresh_from_db()
@@ -510,6 +669,7 @@ class TestController(
         )
         self.assertEqual(response.status_code, 404)
 
+    @capture_any_output()
     def test_device_report_status_400(self):
         d = self._create_device_config()
         response = self.client.post(
@@ -528,6 +688,7 @@ class TestController(
         self.assertEqual(response.status_code, 400)
         self._check_header(response)
 
+    @capture_any_output()
     def test_device_report_status_403(self):
         d = self._create_device_config()
         response = self.client.post(
@@ -552,6 +713,7 @@ class TestController(
 
     def test_device_update_info(self):
         d = self._create_device_config()
+        url = reverse('controller:device_update_info', args=[d.pk])
         params = {
             'key': d.key,
             'model': 'TP-Link TL-WDR4300 v2',
@@ -561,9 +723,7 @@ class TestController(
         self.assertNotEqual(d.os, params['os'])
         self.assertNotEqual(d.system, params['system'])
         self.assertNotEqual(d.model, params['model'])
-        response = self.client.post(
-            reverse('controller:device_update_info', args=[d.pk]), params
-        )
+        response = self.client.post(url, params)
         self.assertEqual(response.status_code, 200)
         self._check_header(response)
         d.refresh_from_db()
@@ -571,6 +731,20 @@ class TestController(
         self.assertEqual(d.system, params['system'])
         self.assertEqual(d.model, params['model'])
 
+        with self.subTest('ignore empty values'):
+            response = self.client.post(
+                url, {'key': d.key, 'model': '', 'os': '', 'system': ''}
+            )
+            self.assertEqual(response.status_code, 200)
+            self._check_header(response)
+            d.refresh_from_db()
+            self.assertNotEqual(d.os, '')
+            self.assertNotEqual(d.system, '')
+            self.assertNotEqual(d.model, '')
+            self.assertEqual(d.os, params['os'])
+            self.assertEqual(d.system, params['system'])
+            self.assertEqual(d.model, params['model'])
+
     def test_device_update_info_bad_uuid(self):
         d = self._create_device_config()
         pk = '{}-wrong'.format(d.pk)
@@ -601,6 +775,7 @@ class TestController(
         self.assertEqual(response.status_code, 400)
         self._check_header(response)
 
+    @capture_any_output()
     def test_device_update_info_403(self):
         d = self._create_device_config()
         params = {
@@ -763,6 +938,7 @@ class TestController(
         self.assertEqual(d.config.templates.filter(pk=t_shared.pk).count(), 1)
         self.assertEqual(d.config.templates.filter(pk=t2.pk).count(), 0)
 
+    @capture_any_output()
     def test_register_400(self):
         self._get_org()
         # missing secret
@@ -811,6 +987,7 @@ class TestController(
         self.assertContains(response, 'mac_address', status_code=400)
         self._check_header(response)
 
+    @capture_any_output()
     def test_register_403(self):
         self._get_org()
         # wrong secret
@@ -839,6 +1016,7 @@ class TestController(
         self.assertContains(response, 'wrong backend', status_code=403)
         self._check_header(response)
 
+    @capture_any_output()
     def test_register_403_disabled_registration(self):
         org = self._get_org()
         org.config_settings.registration_enabled = False
@@ -859,6 +1037,7 @@ class TestController(
         ).count()
         self.assertEqual(count, 0)
 
+    @capture_any_output()
     def test_register_403_disabled_org(self):
         self._create_org(is_active=False)
         response = self.client.post(
@@ -902,7 +1081,7 @@ class TestController(
         org = self._get_org()
         c = self._create_config(organization=org)
         response = self.client.get(
-            reverse('controller:device_checksum', args=[c.device.pk]),
+            reverse('controller:device_checksum', args=[c.device.pk.hex]),
             {'key': c.device.key},
         )
         self.assertEqual(response.status_code, 200)
@@ -925,6 +1104,7 @@ class TestController(
         ).count()
         self.assertEqual(count, 0)
 
+    @patch.object(app_settings, 'SHARED_MANAGEMENT_IP_ADDRESS_SPACE', False)
     def test_ip_fields_not_duplicated(self):
         org1 = self._get_org()
         c1 = self._create_config(organization=org1)
@@ -934,28 +1114,72 @@ class TestController(
         c2 = self._create_config(device=d2)
         org2 = self._create_org(name='org2', shared_secret='123456')
         c3 = self._create_config(organization=org2)
-        self.client.get(
-            reverse('controller:device_checksum', args=[c3.device.pk]),
-            {'key': c3.device.key, 'management_ip': '192.168.1.99'},
-        )
-        self.client.get(
-            reverse('controller:device_checksum', args=[c1.device.pk]),
-            {'key': c1.device.key, 'management_ip': '192.168.1.99'},
-        )
-        self.client.get(
-            reverse('controller:device_checksum', args=[c2.device.pk]),
-            {'key': c2.device.key, 'management_ip': '192.168.1.99'},
-        )
+        with self.assertNumQueries(6):
+            self.client.get(
+                reverse('controller:device_checksum', args=[c3.device.pk]),
+                {'key': c3.device.key, 'management_ip': '192.168.1.99'},
+            )
+        with self.assertNumQueries(6):
+            self.client.get(
+                reverse('controller:device_checksum', args=[c1.device.pk]),
+                {'key': c1.device.key, 'management_ip': '192.168.1.99'},
+            )
+        with self.assertNumQueries(0):
+            # repeat the request to test the checksum view cache interaction
+            self.client.get(
+                reverse('controller:device_checksum', args=[c1.device.pk]),
+                {'key': c1.device.key, 'management_ip': '192.168.1.99'},
+            )
+        # triggers more queries because devices with conflicting addresses
+        # need to be updated, luckily it does not happen often
+        with self.assertNumQueries(8):
+            self.client.get(
+                reverse('controller:device_checksum', args=[c2.device.pk]),
+                {'key': c2.device.key, 'management_ip': '192.168.1.99'},
+            )
         c1.refresh_from_db()
         c2.refresh_from_db()
         c3.refresh_from_db()
         # device previously having the IP now won't have it anymore
         self.assertNotEqual(c1.device.last_ip, c2.device.last_ip)
         self.assertNotEqual(c1.device.management_ip, c2.device.management_ip)
+        self.assertIsNone(c1.device.management_ip)
+        self.assertEqual(c2.device.management_ip, '192.168.1.99')
         # other organization is not affected
         self.assertEquals(c3.device.last_ip, '127.0.0.1')
         self.assertEqual(c3.device.management_ip, '192.168.1.99')
 
+        with self.subTest('test interaction with DeviceChecksumView caching'):
+            view = DeviceChecksumView()
+            view.kwargs = {'pk': str(c1.device.pk)}
+            cached_device1 = view.get_device()
+            self.assertIsNone(cached_device1.management_ip)
+
+    @patch.object(app_settings, 'SHARED_MANAGEMENT_IP_ADDRESS_SPACE', True)
+    def test_organization_shares_management_ip_address_space(self):
+        org1 = self._get_org()
+        org1_config = self._create_config(organization=org1)
+        org2 = self._create_org(name='org2', shared_secret='org2')
+        org2_config = self._create_config(organization=org2)
+        with self.assertNumQueries(6):
+            self.client.get(
+                reverse('controller:device_checksum', args=[org1_config.device_id]),
+                {'key': org1_config.device.key, 'management_ip': '192.168.1.99'},
+            )
+        # Device from another organization sends conflicting management IP
+        # Extra queries due to conflict resolution
+        with self.assertNumQueries(8):
+            self.client.get(
+                reverse('controller:device_checksum', args=[org2_config.device_id]),
+                {'key': org2_config.device.key, 'management_ip': '192.168.1.99'},
+            )
+        org1_config.refresh_from_db()
+        org2_config.refresh_from_db()
+        # device previously having the IP now won't have it anymore
+        self.assertIsNone(org1_config.device.management_ip)
+        self.assertEqual(org2_config.device.management_ip, '192.168.1.99')
+        self.assertNotEqual(org1_config.device.last_ip, org2_config.device.last_ip)
+
     # simulate public IP by mocking the
     # method which tells us if the ip is private or not
     @patch('ipaddress.IPv4Address.is_private', False)
