@@ -3,7 +3,8 @@ from unittest import mock
 
 from celery.exceptions import SoftTimeLimitExceeded
 from django.contrib.auth import get_user_model
-from django.core.exceptions import ValidationError
+from django.core.exceptions import PermissionDenied, ValidationError
+from django.db import transaction
 from django.test import TestCase, TransactionTestCase
 from netjsonconfig import OpenWrt
 from swapper import load_model
@@ -65,7 +66,7 @@ class TestTemplate(
             c.templates.add(t)
             c.refresh_from_db()
             handler.assert_called_once_with(
-                sender=Config, signal=config_status_changed, instance=c,
+                sender=Config, signal=config_status_changed, instance=c
             )
 
     def test_no_auto_hostname(self):
@@ -328,8 +329,8 @@ class TestTemplate(
         organization = self._get_org()
         vpn = self._create_vpn()
         template = self._create_template(type='vpn', auto_cert=True, vpn=vpn)
-        corresponding_device = self._create_device(organization=organization,)
-        config = self._create_config(device=corresponding_device,)
+        corresponding_device = self._create_device(organization=organization)
+        config = self._create_config(device=corresponding_device)
         config.templates.add(template)
         vpn_clients = config.vpnclient_set.all()
         for vpn_client in vpn_clients:
@@ -356,7 +357,7 @@ class TestTemplate(
             template_qs = Template.objects.filter(type='vpn')
             self.assertEqual(template_qs.count(), 1)
             t = template_qs.first()
-            self.assertEqual(t.get_context(), _original_context)
+            self.assertDictContainsSubset(_original_context, t.get_context())
             self.assertEqual(app_settings.CONTEXT, _original_context)
 
         with self.subTest(
@@ -380,6 +381,127 @@ class TestTemplate(
         system_context = t.get_system_context()
         self.assertNotIn('test', system_context.keys())
 
+    def test_template_name_unique_validation(self):
+        org = self._get_org()
+        template1 = self._create_template(name='test', organization=org)
+        self.assertEqual(template1.name, 'test')
+        org2 = self._create_org(name='test org2', slug='test-org2')
+
+        with self.subTest('template of other org can have the same name'):
+            try:
+                template2 = self._create_template(name='test', organization=org2)
+            except Exception as e:
+                self.fail(f'Unexpected exception: {e}')
+            self.assertEqual(template2.name, 'test')
+
+        with self.subTest('template of shared org cannot have the same name'):
+            with self.assertRaises(ValidationError) as context_manager:
+                self._create_template(name='test', organization=None)
+            message_dict = context_manager.exception.message_dict
+            self.assertIn('name', message_dict)
+            self.assertIn(
+                'There is already a template of another organization',
+                message_dict['name'][0],
+            )
+
+        with self.subTest(
+            'new template of org cannot have the same name as shared template'
+        ):
+            shared = self._create_template(name='new', organization=None)
+            with self.assertRaises(ValidationError) as context_manager:
+                self._create_template(name='new', organization=org2)
+            message_dict = context_manager.exception.message_dict
+            self.assertIn('name', message_dict)
+            self.assertIn(
+                'There is already another shared template', message_dict['name'][0]
+            )
+
+        with self.subTest('ensure object itself is excluded'):
+            try:
+                shared.full_clean()
+            except Exception as e:
+                self.fail(f'Unexpected exception {e}')
+
+        with self.subTest('cannot have two shared templates with same name'):
+            with self.assertRaises(ValidationError) as context_manager:
+                self._create_template(name='new', organization=None)
+            message_dict = context_manager.exception.message_dict
+            self.assertIn('name', message_dict)
+
+    def test_required_templates(self):
+        dummy_config = {'interfaces': []}
+        t4 = self._create_template(name='default4', config=dummy_config, default=True)
+        t2 = self._create_template(name='required2', config=dummy_config, required=True)
+        t1 = self._create_template(name='required1', config=dummy_config, required=True)
+        t3 = self._create_template(name='default3', config=dummy_config, default=True)
+
+        with self.subTest('required makes it also default'):
+            self.assertTrue(t1.default)
+            t1.default = False
+            t1.full_clean()
+            self.assertTrue(t1.default)
+
+        with self.subTest('test required templates ordering'):
+            config = self._create_config(
+                device=self._create_device(name='test-required')
+            )
+            self.assertTrue(config.templates.filter(pk=t1.pk).exists())
+            self.assertTrue(config.templates.filter(pk=t2.pk).exists())
+            templates = config.templates.all()
+            self.assertEqual(templates[0], t1)
+            self.assertEqual(templates[1], t2)
+            self.assertEqual(templates[2], t3)
+            self.assertEqual(templates[3], t4)
+
+        with self.subTest('removing a required template from a device raises error'):
+            with transaction.atomic():
+                with self.assertRaises(PermissionDenied) as context:
+                    config.templates.remove(t1)
+            self.assertIn('Required templates', str(context.exception))
+
+        with self.subTest('clearing required templates is ineffective (sortedm2m)'):
+            t5 = self._create_template(name='not-required', required=False)
+            config.templates.clear()
+            config.templates.add(t5)
+            self.assertTrue(config.templates.filter(pk=t1.pk).exists())
+            self.assertTrue(config.templates.filter(pk=t2.pk).exists())
+
+        with self.subTest('required template honours backend of config'):
+            config.backend = 'netjsonconfig.OpenWisp'
+            config.save()
+            self.assertEqual(config.templates.count(), 3)
+
+            # This should not raise an exception since backends of
+            # t5 and config is not same. Also t5 should not be added back
+            config.templates.remove(t1)
+            self.assertEqual(config.templates.count(), 2)
+
+            # Similarly, clearing all templates should be possible
+            config.templates.clear()
+            self.assertEqual(config.templates.count(), 0)
+
+    def test_required_vpn_template_corner_case(self):
+        org = self._get_org()
+        vpn = self._create_vpn()
+        t = self._create_template(
+            name='vpn-test',
+            type='vpn',
+            vpn=vpn,
+            auto_cert=True,
+            required=True,
+            default=True,
+        )
+        c = self._create_config(organization=org)
+        vpn_client = c.vpnclient_set.first()
+        self.assertIsNotNone(vpn_client)
+        # simulate reordering via sortedm2m
+        c.templates.clear()
+        c.templates.add(t)
+        # ensure no error is raised
+        # ValidationError:
+        # {'__all__': ['VPN client with this Config and Vpn already exists.']}
+        self.assertIsNotNone(vpn_client)
+
 
 class TestTemplateTransaction(
     TestOrganizationMixin,
@@ -409,72 +531,138 @@ class TestTemplateTransaction(
                 t.save()
                 c.refresh_from_db()
                 handler.assert_called_once_with(
-                    sender=Config, signal=config_status_changed, instance=c,
+                    sender=Config, signal=config_status_changed, instance=c
                 )
                 self.assertEqual(c.status, 'modified')
 
-        with self.subTest(
-            'signal not sent if config is already modified (additional case)'
-        ):
+        with self.subTest('signal not sent if config is already modified'):
             # status has already changed to modified
             # sgnal should not be triggered again
             with catch_signal(config_status_changed) as handler:
                 t.config['interfaces'][0]['name'] = 'eth2'
                 t.full_clean()
-                with self.assertNumQueries(7):
+                with self.assertNumQueries(8):
                     t.save()
                 c.refresh_from_db()
                 handler.assert_not_called()
                 self.assertEqual(c.status, 'modified')
 
     def test_config_modified_signal(self):
-        temp = self._create_template()
         conf = self._create_config(device=self._create_device(name='test-status'))
+        template1 = self._create_template(name='t1')
+        template2 = self._create_template(
+            name='t2', organization=conf.device.organization
+        )
         self.assertEqual(conf.status, 'modified')
         # refresh instance to reset _just_created attribute
         conf = Config.objects.get(pk=conf.pk)
 
-        with self.subTest('signal not sent m2m if config status is already modified'):
-            # (avoids executing push updates multiple times)
+        with self.subTest('signal sent if config status is already modified'):
             with catch_signal(config_modified) as handler:
-                conf.templates.add(temp)
-                handler.assert_not_called()
+                conf.templates.add(template1, template2)
+                handler.assert_called_with(
+                    sender=Config,
+                    signal=config_modified,
+                    instance=conf,
+                    device=conf.device,
+                    config=conf,
+                    previous_status='modified',
+                    action='m2m_templates_changed',
+                )
+            self.assertEqual(handler.call_count, 1)
+
+        # reset status
+        conf.set_status_applied()
 
         with self.subTest('signal sent after assigning template to config'):
             with catch_signal(config_modified) as handler:
-                conf.set_status_applied()
-                conf.templates.add(temp)
+                conf.templates.remove(template2)
                 handler.assert_called_once_with(
                     sender=Config,
                     signal=config_modified,
                     instance=conf,
                     device=conf.device,
                     config=conf,
+                    previous_status='applied',
+                    action='m2m_templates_changed',
                 )
+            conf.refresh_from_db()
+            self.assertEqual(conf.status, 'modified')
 
-        conf.status = 'applied'
-        conf.save()
-        conf.refresh_from_db()
-        self.assertEqual(conf.status, 'applied')
-        temp.config['interfaces'][0]['name'] = 'eth1'
+        # reset status
+        conf.set_status_applied()
+
+        with self.subTest('post_clear m2m ignored'):
+            with catch_signal(config_modified) as handler:
+                conf.templates.clear()
+                handler.assert_not_called()
+            conf.refresh_from_db()
+            self.assertEqual(conf.status, 'applied')
+
+        with self.subTest('post_add and previous_status=applied'):
+            with catch_signal(config_modified) as handler:
+                conf.templates.add(template1, template2)
+                handler.assert_called_once_with(
+                    sender=Config,
+                    signal=config_modified,
+                    instance=conf,
+                    device=conf.device,
+                    config=conf,
+                    previous_status='applied',
+                    action='m2m_templates_changed',
+                )
+            self.assertEqual(handler.call_count, 1)
+            conf.refresh_from_db()
+            self.assertEqual(conf.status, 'modified')
+
+        # reset status
+        conf.set_status_applied()
+
+        with self.subTest('test remove/add'):
+            with catch_signal(config_modified) as handler:
+                conf.templates.remove(template1, template2)
+                conf.templates.add(template1, template2)
+                self.assertEqual(handler.call_count, 2)
+            conf.refresh_from_db()
+            self.assertEqual(conf.status, 'modified')
+
+        # reset status
+        conf.set_status_applied()
 
         with self.subTest('signal sent after changing a template'):
             with catch_signal(config_modified) as handler:
-                temp.full_clean()
-                temp.save()
+                template1.config['interfaces'][0]['name'] = 'eth1'
+                template1.full_clean()
+                template1.save()
+                handler.assert_called_once_with(
+                    sender=Config,
+                    signal=config_modified,
+                    instance=conf,
+                    device=conf.device,
+                    config=conf,
+                    previous_status='applied',
+                    action='related_template_changed',
+                )
                 conf.refresh_from_db()
-                handler.assert_called_once()
                 self.assertEqual(conf.status, 'modified')
 
         with self.subTest('signal sent also if config is already in modified status'):
             # status has already changed to modified
             # sgnal should be triggered anyway
             with catch_signal(config_modified) as handler:
-                temp.config['interfaces'][0]['name'] = 'eth2'
-                temp.full_clean()
-                temp.save()
+                template1.config['interfaces'][0]['name'] = 'eth2'
+                template1.full_clean()
+                template1.save()
                 conf.refresh_from_db()
-                handler.assert_called_once()
+                handler.assert_called_once_with(
+                    sender=Config,
+                    signal=config_modified,
+                    instance=conf,
+                    device=conf.device,
+                    config=conf,
+                    previous_status='modified',
+                    action='related_template_changed',
+                )
                 self.assertEqual(conf.status, 'modified')
 
     @mock.patch.object(update_template_related_config_status, 'delay')
@@ -485,12 +673,28 @@ class TestTemplateTransaction(
             conf.set_status_applied()
             mocked_task.assert_not_called()
 
-        with self.subTest('task is called when template is assigned to conf'):
+        with self.subTest('task is called when template conf is changed'):
             template.config['interfaces'][0]['name'] = 'eth1'
             template.full_clean()
             template.save()
             mocked_task.assert_called_with(template.pk)
 
+        mocked_task.reset_mock()
+
+        with self.subTest('task is called when template default_values are changed'):
+            template.refresh_from_db()
+            template.default_values = {'a': 'a'}
+            template.full_clean()
+            template.save()
+            mocked_task.assert_called_with(template.pk)
+
+        mocked_task.reset_mock()
+
+        with self.subTest('task is not called when there are no changes'):
+            template.full_clean()
+            template.save()
+            mocked_task.assert_not_called()
+
     @mock.patch.object(task_logger, 'warning')
     def test_task_failure(self, mocked_warning):
         update_template_related_config_status.delay(uuid.uuid4())
