@@ -1,5 +1,5 @@
 from django.core.validators import RegexValidator, _lazy_re_compile
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 key_validator = RegexValidator(
     _lazy_re_compile('^[^\s/\.]+$'),
