@@ -1,6 +1,6 @@
 import swapper
 
-from .base.models import AbstractCredentials, AbstractDeviceConnection
+from .base.models import AbstractCommand, AbstractCredentials, AbstractDeviceConnection
 
 
 class Credentials(AbstractCredentials):
@@ -11,4 +11,11 @@ class Credentials(AbstractCredentials):
 
 class DeviceConnection(AbstractDeviceConnection):
     class Meta(AbstractDeviceConnection.Meta):
+        abstract = False
         swappable = swapper.swappable_setting('connection', 'DeviceConnection')
+
+
+class Command(AbstractCommand):
+    class Meta(AbstractCommand.Meta):
+        abstract = False
+        swappable = swapper.swappable_setting('connection', 'Command')
